#
# spec file for package llvm
#
# Copyright (c) 2012 SUSE LINUX Products GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%define _revision     145598
%define _release_version 3.0
%define _supported_archs "arm,x86"

Name:           llvm
Version:        3.0
Release:        1.9.13
Summary:        Low Level Virtual Machine
License:        NCSA
Group:          Development/Languages/Other
Url:            http://www.llvm.org
Source0:        %{name}-%{version}.tar.bz2
Source100:      %{name}-rpmlintrc
# PATCH-FIX-OPENSUSE set-revision.patch idoenmez@suse.de -- Allow us to set revision
Patch1:         set-revision.patch
# PATCH-FIX-OPENSUSE assume-opensuse.patch idoenmez@suse.de -- Always enable openSUSE/SUSE features
Patch3:         assume-opensuse.patch
# PATCH-FIX-UPSTREAM clang-glibc2.14.patch idoenmez@suse.de -- Support glibc 2.14 headers
Patch4:         clang-glibc2.14.patch
# PATCH-FIX-OPENSUSE clang-disable-ada-extension.patch idoenmez@suse.de -- Don't run gcc for ada files
Patch5:         clang-disable-ada-extension.patch
# PATCH-FIX-UPSTREAM http://llvm.org/viewvc/llvm-project?view=rev&revision=147295
Patch6:         clang-pr11642.patch
# PATCH-FIX-OPENSUSE default-to-i586.patch -- Use i586 as default target for 32bit
Patch7:         default-to-i586.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
BuildRequires:  autoconf
BuildRequires:  automake
%if 0%{?suse_version} >= 1220
BuildRequires:  binutils-devel >= 2.21.90
%endif
BuildRequires:  bison
BuildRequires:  chrpath
BuildRequires:  dejagnu
BuildRequires:  fdupes
BuildRequires:  flex
BuildRequires:  gcc
BuildRequires:  gcc-c++
BuildRequires:  libtool
BuildRequires:  python-devel
BuildRequires:  tcl-devel
BuildRequires:  tk-devel

# Code10 does not have libffi
%if 0%{?suse_version} > 1100
BuildRequires:  libffi-devel
%endif

# LLVM is not supported on PPC64
# http://llvm.org/bugs/show_bug.cgi?id=3729
ExcludeArch:    ppc64

%description
LLVM is a compiler infrastructure designed for compile-time,
link-time, runtime, and idle-time optimization of programs from
arbitrary programming languages.

The compiler infrastructure includes mirror sets of programming
tools as well as libraries with equivalent functionality.

%package devel

Summary:        Libraries and Header Files for LLVM
Group:          Development/Languages/Other
Requires:       %{name} = %{version}
Requires:       libffi-devel
Requires:       libstdc++-devel >= 3.4

%description devel
This package contains library and header files needed to develop
new native programs that use the LLVM infrastructure.

%package doc

Summary:        Documentation for LLVM
Group:          Documentation/HTML
Requires:       %{name} = %{version}

%description doc
Documentation for the LLVM compiler infrastructure.

%package clang

Summary:        CLANG frontend for LLVM
Group:          Development/Languages/Other
Requires:       %{name} = %{version}
Provides:       clang
%{py_requires}

%description clang
This package contains the clang (C language) frontend for LLVM.

%package clang-devel

Summary:        CLANG frontend for LLVM (devel package)
Group:          Development/Languages/Other
Requires:       clang = %{version}
Requires:       llvm-devel = %{version}
%{py_requires}

%description clang-devel
This package contains the clang (C language) frontend for LLVM.
(development files)

%package        vim-plugins

Summary:        Vim plugins for LLVM
Group:          Productivity/Text/Editors
Requires:       vim

%description    vim-plugins
This package contains vim plugins for LLVM like syntax highlighting.

%prep
%setup -q
%patch1
%patch3
cd tools/clang
%patch4
%patch6
cd ../..
%patch5 -p1
%patch7 -p1

# We hardcode i586
rm tools/clang/test/Driver/x86_features.c

sed -i s,SVN_REVISION,\"%{_revision}\",g tools/clang/lib/Basic/Version.cpp

# Nasty hardcoded path
sed -i s,/lib/,/%{_lib}/,g tools/clang/lib/Driver/Tools.cpp

%build
TOPLEVEL=$PWD
mkdir stage1 stage2 stage3
pushd stage1

../configure \
	--enable-optimized \
	--disable-assertions \
	--with-optimize-option="-O0" \
	--enable-targets=host

make %{?_smp_mflags}

popd
pushd stage2

export CC=$TOPLEVEL/stage1/Release/bin/clang
export CXX=$TOPLEVEL/stage1/Release/bin/clang++

../configure \
	--enable-optimized \
	--disable-assertions \
	--enable-targets=host

make %{?_smp_mflags}

popd
pushd stage3

# autotools ignore --libdir
mkdir Release
ln -s lib Release/lib64

export CC=$TOPLEVEL/stage2/Release/bin/clang
export CXX=$TOPLEVEL/stage2/Release/bin/clang++

../configure \
        --prefix=%{_prefix} \
	--enable-optimized \
	--disable-assertions \
	--enable-pic \
	--disable-timestamps \
        --enable-targets=%{_supported_archs} \
%if 0%{?suse_version} >= 1220
	--with-binutils-include=/usr/include \
%endif
	--with-clang-resource-dir="../%{_lib}/clang/%{_release_version}"

make %{?_smp_mflags} REQUIRES_RTTI=1

%check
cd stage3
make check
cd tools/clang
make test

%install
cd stage3
%makeinstall
cd ..

# Fixup libdir
%if "%{_lib}" == "lib64"
sed -i s,ABS_RUN_DIR/lib,ABS_RUN_DIR/lib64, %{buildroot}/%{_bindir}/llvm-config
mv %{buildroot}%{_prefix}/lib %{buildroot}%{_prefix}/lib64
%endif

# Docs
mkdir -p  %{buildroot}/usr/share/doc/packages/llvm-doc/
mv %{buildroot}/usr/docs/llvm/html %{buildroot}/usr/share/doc/packages/llvm-doc/
rm -rf %{buildroot}/usr/docs/

# cleanup
file %{buildroot}/%{_bindir}/* | awk -F: '$2~/ELF/{print $1}' | xargs -r chrpath -d
file %{buildroot}/%{_libdir}/%{name}/*.so | awk -F: '$2~/ELF/{print $1}' | xargs -r chrpath -d
find %{buildroot}%{_includedir}/clang -name '*.tmp' -exec rm {} +
chmod -x %{buildroot}/%{_libdir}/*.a

for file in `find examples -type f`; do
	install -d %{buildroot}%{_docdir}/%{name}-doc/`dirname $file`
	install -m 644 $file %{buildroot}%{_docdir}/%{name}-doc/$file
done

find %{buildroot}%{_docdir}/%{name}-doc/ -name '*.tar.gz' -exec rm {} \;

# install scan-build et al
install -d %{buildroot}%{_datadir}/clang
install -d %{buildroot}%{py_sitedir}

pushd tools/clang/tools
for file in scan-build/ccc-analyzer scan-build/scan-build \
		scan-view/scan-view; do
	install $file %{buildroot}%{_bindir}
done
for file in scan-build/scanview.css scan-build/sorttable.js; do
	install -m 644 $file %{buildroot}%{_datadir}/clang/
done
for file in scan-view/Reporter.py scan-view/ScanView.py \
		scan-view/startfile.py; do
	install -m 644 $file %{buildroot}%{py_sitedir}
done
install -d %{buildroot}%{py_sitedir}/Resources/
for file in scan-view/Resources/*; do
	install -m 644 $file %{buildroot}%{py_sitedir}/Resources/
done
popd
pushd %{buildroot}%{_bindir}
ln -s ccc-analyzer c++-analyzer
popd

sed -i 's,RealBin/bin/clang,RealBin/clang,' \
  %{buildroot}%{_bindir}/scan-build

sed -i 's,RealBin/bin/ccc-analyzer,RealBin/ccc-analyzer,' \
  %{buildroot}%{_bindir}/scan-build

sed -i 's,RealBin/bin/ccc-analyzer,RealBin/ccc-analyzer,' \
  %{buildroot}%{_bindir}/scan-build

sed -i 's,$RealBin/sorttable.js,%{_datadir}/clang/sorttable.js,' \
  %{buildroot}%{_bindir}/scan-build

sed -i 's,$RealBin/scanview.css,%{_datadir}/clang/scanview.css,' \
  %{buildroot}%{_bindir}/scan-build

sed -i 's,RealBin/bin,RealBin,' \
  %{buildroot}%{_bindir}/scan-build

install -m 755 -d %{buildroot}%{_datadir}/vim/site/syntax/
install -m 644 utils/vim/*.vim %{buildroot}%{_datadir}/vim/site/syntax/
mv utils/vim/README utils/vim/README.vim

# Remove gtest library
rm -f %{buildroot}%{_libdir}/*gtest*

# Wrong lib name + static
rm -f %{buildroot}%{_libdir}/liblibclang.a

# Stuff we don't want to include
rm -f %{buildroot}%{_libdir}/BugpointPasses.*
rm -f %{buildroot}%{_libdir}/LLVMHello.*
rm -f %{buildroot}%{_libdir}/libCompilerDriver.*
rm -f %{buildroot}%{_libdir}/libEnhancedDisassembly.*

%fdupes -s %{buildroot}%{_docdir}/%{name}
%fdupes -s %{buildroot}%{_docdir}/%{name}-doc

%post clang -p /sbin/ldconfig                                                                               

%postun clang -p /sbin/ldconfig

%post clang-devel -p /sbin/ldconfig

%postun clang-devel -p /sbin/ldconfig

%files
%defattr(-,root,root)
%doc *.TXT *.txt
%{_bindir}/bugpoint
%{_bindir}/llc
%{_bindir}/lli
%{_bindir}/macho-dump
%exclude %{_bindir}/%{name}-config
%{_bindir}/%{name}*
%{_bindir}/opt
%{_mandir}/man1/bugpoint.1%{ext_man}
%exclude %{_mandir}/man1/lit.1%{ext_man}
%{_mandir}/man1/tblgen.1%{ext_man}
%{_mandir}/man1/llc.1%{ext_man}
%{_mandir}/man1/lli.1%{ext_man}
%exclude %{_mandir}/man1/%{name}-config.1%{ext_man}
%{_mandir}/man1/llvm*.1%{ext_man}
%{_mandir}/man1/opt.1%{ext_man}

%files clang
%defattr(-,root,root)
%{_bindir}/clang
%{_bindir}/clang-*
%{_bindir}/clang++
%{_bindir}/c-index-test
%{_bindir}/ccc-analyzer
%{_bindir}/c++-analyzer
%{_bindir}/scan-build
%{_bindir}/scan-view
%{py_sitedir}/ScanView.py
%{py_sitedir}/Reporter.py
%{py_sitedir}/startfile.py
%{py_sitedir}/Resources
%{_datadir}/clang
%{_mandir}/man1/clang.1.gz
%{_libdir}/libclang.so
%{_libdir}/libLTO.so
%{_libdir}/*profile*.a
%if 0%{?suse_version} >= 1220
%{_libdir}/LLVMgold.so
%endif
%dir %{_libdir}/clang
%dir %{_libdir}/clang/*
%dir %{_libdir}/clang/*/include
%{_libdir}/clang/*/include/altivec.h
%{_libdir}/clang/*/include/arm_neon.h
%{_libdir}/clang/*/include/avxintrin.h
%{_libdir}/clang/*/include/emmintrin.h
%{_libdir}/clang/*/include/float.h
%{_libdir}/clang/*/include/immintrin.h
%{_libdir}/clang/*/include/iso646.h
%{_libdir}/clang/*/include/limits.h
%{_libdir}/clang/*/include/mm_malloc.h
%{_libdir}/clang/*/include/mmintrin.h
%{_libdir}/clang/*/include/nmmintrin.h
%{_libdir}/clang/*/include/pmmintrin.h
%{_libdir}/clang/*/include/smmintrin.h
%{_libdir}/clang/*/include/stdalign.h
%{_libdir}/clang/*/include/stdarg.h
%{_libdir}/clang/*/include/stdbool.h
%{_libdir}/clang/*/include/stddef.h
%{_libdir}/clang/*/include/stdint.h
%{_libdir}/clang/*/include/tgmath.h
%{_libdir}/clang/*/include/tmmintrin.h
%{_libdir}/clang/*/include/varargs.h
%{_libdir}/clang/*/include/wmmintrin.h
%{_libdir}/clang/*/include/x86intrin.h
%{_libdir}/clang/*/include/xmmintrin.h
%{_libdir}/clang/*/include/mm3dnow.h

%files clang-devel
%defattr(-,root,root)
%{_includedir}/clang
%{_includedir}/clang-c
%{_libdir}/libLTO.a
%{_libdir}/libclang*.a
%{_libdir}/*profile*.so

%files devel
%defattr(-,root,root,-)
%{_bindir}/%{name}-config
%doc %{_mandir}/man1/%{name}-config.1%{ext_man}
%{_includedir}/%{name}/
%{_includedir}/%{name}-c/
%{_libdir}/libLLVM*

%files doc
%defattr(-,root,root,-)
%{_docdir}/%{name}-doc

%files vim-plugins
%defattr(-,root,root,-)
%doc utils/vim/README.vim
%{_datadir}/vim/

%changelog
* Tue Mar 20 2012 idonmez@suse.com
- Target i586-linux for 32bit builds  (bnc#728737)
* Tue Dec 27 2011 idonmez@suse.com
- Add upstream fix for llvm PR11642
* Sun Dec 25 2011 idonmez@suse.com
- Don't run gcc for ada files, just fail instead.
* Thu Dec 22 2011 idonmez@suse.com
- Add clang-glibc2.14.patch to support glibc 2.14 headers
- Install profiling support
* Mon Dec  5 2011 idoenmez@suse.de
- Switch to 3-stage bootstrap
* Fri Dec  2 2011 idoenmez@suse.de
- Build first stage compiler with -O0 to workaround gcc 4.5 bug
* Thu Dec  1 2011 idoenmez@suse.de
- Update to final 3.0 release
  * No real change since rc2
* Tue Nov 29 2011 idoenmez@suse.de
- Use %%{_lib} macro instead of checking for x86_64
* Wed Nov 16 2011 idonmez@suse.com
- Enable ARM support again
- Switch to optimized build
* Thu Nov 10 2011 idonmez@suse.com
- Disable ARM support, VM goes out of memory while compiling it
* Thu Nov 10 2011 idonmez@suse.com
- Really enable ARM support
- Remove forcing -frtti
* Mon Nov  7 2011 idonmez@suse.com
- Update to r143961 from llvm-3.0 branch
  * No longer hardcoding include paths
* Mon Nov  7 2011 idonmez@suse.com
- Fix LLVMgold.so path for x86-64
* Sun Nov  6 2011 idonmez@suse.com
- Enable ld gold support
* Tue Nov  1 2011 idonmez@suse.com
- Update r143456 from llvm-3.0 branch
  * llvm/clang 3.0rc2
* Fri Oct 28 2011 idonmez@suse.com
- Add llvm-pr9614-part{1,2}.patch to fix LLVM PR9614
* Fri Oct 28 2011 idonmez@suse.com
- Fixup dependencies so that llvm-clang doesn't depend
  on llvm-clang-devel
* Tue Oct 25 2011 idonmez@suse.com
- Update to r142912 llvm-3.0 branch
  * Fixes to python bindings
* Fri Oct 21 2011 idonmez@suse.com
- Update to r142641
  * llvm/clang 3.0 rc1
* Thu Oct 20 2011 idonmez@suse.com
- Enable ARM code generation
* Wed Oct 19 2011 idonmez@suse.com
- Fix libdir on x86_64
* Tue Oct 18 2011 idonmez@suse.com
- Update to r142341
  * First build from 3.0 branch
  * -Wc++98-compat warnings for the lexer
  * Add flags for the remaining shift related warnings
    (-Wshift-count-negative, -Wshift-count-overflow).
  * Only warn in -Wliteral-conversion if the conversion
    loses information
  * Added clang_getCompletionAnnotation and
    clang_getCompletionNumAnnotations to retrieve annotations
    from completion string.
  * Add support for -std=gnu90 and -std=c++03, for compatibility
    with modern gcc.
- Switch to tar.bz2 and remove xz dependency
* Thu Sep 29 2011 idonmez@suse.com
- Update to r140780
  * Code cleanup
  * Crash fixes
  * Add support for alignment-specifiers in C1X and C++11
  * Introduce a new compiler switch: -fms-compatility
  * Emit debug info for c++0x nullptr.
* Tue Sep 13 2011 idonmez@suse.com
- Update to r139589
  * C++0x fixes
  * PCH fixes
* Thu Sep  1 2011 idonmez@suse.com
- Update to r138939
  * Many crash fixes.
  * Performance optimizations.
  * Allow C99 hexfloats in C++0x mode.
  * Add missing function _mm_ucomige_sd to emmintrin.h.
* Fri Aug 12 2011 idonmez@novell.com
- Update to r137452
  * Fix problems found by PVS-Studio
  * More AVX fixes
  * Many C++ fixes
- Add assume-opensuse.patch to remove dependency on openSUSE-release
* Wed Aug  3 2011 idonmez@novell.com
- Update to r136773
  * Many AVX fixes
  * Support for C++0x unicode string and character literals
* Tue Jul 19 2011 vljn@ovi.com
- Update to r135431
  * API changes to match Mesa requirements
* Mon Jul 11 2011 idonmez@novell.com
- Update to r134888
  * Type system rewrite
* Sat Jul  9 2011 idonmez@novell.com
- Update to r134813
  * Mainly gcc compatibility fixes
* Fri Jul  8 2011 idonmez@novell.com
- Update to r134698
  * Up to 80x speed improvements with -Wuninitialized
* Thu Jul  7 2011 idonmez@novell.com
- Enable RTTI support
- Disable assertions because they are not thread safe
- Update to r134611
  * Many C++ fixes and speedups
  * Updates for AVX instruction support
  * gcc 4.6.1 support
* Sat Jul  2 2011 idonmez@novell.com
-  Fix manpage generation (bnc #703591)
* Sun Jun  5 2011 idonmez@novell.com
- Update to r132667
* Sun May 29 2011 idonmez@novell.com
- Update to use cmake buildsystem
- Remove unmaintained ocaml support
* Thu May 26 2011 idonmez@novell.com
- Drop ocaml support because its unmaintained
- Update to latest svn revision 132050
  + Start of the upcoming 3.0 version
  + Support for gcc 4.6 c++ headers
  + Better support for C++0x
* Fri Apr 29 2011 idoenmez@novell.com
- Fix ocaml dependency
* Thu Apr  7 2011 idoenmez@novell.com
- Update to 2.9
  + Type Based Alias Analysis (TBAA) is now implemented in clang
  + LLVM now generates better debug information
  + Inline assembly now supports multiple alternative constraints
* Wed Apr  6 2011 stepan@coresystems.de
- drop revision from Requires: fields
- update to 2.8.99 revision 129008.
* Mon Mar  7 2011 jslaby@suse.de
- spec file cleanup
* Wed Dec 22 2010 dmacvicar@suse.de
- Do not require specific ffi version but gather the
  appropiate one from the distro
* Fri Dec 10 2010 reddwarf@opensuse.org
- Fix compatibility with C++0x compilers (g++ 4.5)
* Mon Nov 29 2010 ismail@namtrac.org
- Fix C include paths
* Fri Nov 26 2010 ismail@namtrac.org
- Fix header path for openSUSE 11.4
- Enabled PIC for x86 for loadable module support
- Enable package tests for llvm and clang
- Disable package timestamps to silence an rpmlint warning
* Wed Jun  2 2010 anschneider@exsuse.de
- Fixed the build on x86 plattforms.
- Package the clang c++-analyzer.
* Sat May  1 2010 jslaby@suse.de
- update to llvm 2.7
* Mon Mar 15 2010 jslaby@suse.de
- added vim plugins
* Sun Mar 14 2010 jslaby@suse.de
- added libffi-devel to llvm-devel requires so that build does not fail
- fixed build of explicit constructors with gcc45
* Mon Nov  2 2009 jslaby@suse.de
- Initial 2.6 release
